#!/bin/bash

###############################################################################
#                                                                             #
#  Copyright (c) 2010 Nemzeti Információs Infrastruktúra Fejlesztési Intézet  #
#                                                                             #
#  Licensed under the Apache License, Version 2.0 (the "License");            #
#  you may not use this file except in compliance with the License.           #
#  You may obtain a copy of the License at                                    #
#                                                                             #
#    http://www.apache.org/licenses/LICENSE-2.0                               #
#                                                                             #
#  Unless required by applicable law or agreed to in writing, software        #
#  distributed under the License is distributed on an "AS IS" BASIS,          #
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
#  See the License for the specific language governing permissions and        #
#  limitations under the License.                                             #
#                                                                             #
###############################################################################

#set -e

SRC=$1
DST=$2
SIZE=$3

if [ -z "${ONE_LOCATION}" ]; then
    TMCOMMON=/usr/lib/one/mads/tm_common.sh
    LIB=/usr/lib/one/tm_commands/hostiscsi+nfs/iscsi-functions
    ONE_DD=/usr/lib/one/tm_commands/iscsi-common/one_dd
#    RESIZE=/usr/lib/one/tm_commands/iscsi-common/image_resize
    RC=/etc/one/tm_hostiscsi+nfs/tm_hostiscsi+nfsrc
    SI=/usr/lib/ruby/si/cloud_si
else
    TMCOMMON=$ONE_LOCATION/lib/mads/tm_common.sh
    LIB=$ONE_LOCATION/lib/tm_commands/hostiscsi+nfs/iscsi-functions
    ONE_DD=$ONE_LOCATION/lib/tm_commands/iscsi-common/one_dd
#    RESIZE=$ONE_LOCATION/lib/tm_commands/iscsi-common/image_resize
    RC=$ONE_LOCATION/etc/tm_hostiscsi+nfs/tm_hostiscsi+nfsrc
    SI=$ONE_LOCATION/lib/ruby/si/cloud_si
    PATH=$ONE_LOCATION/bin:$PATH
fi

. $TMCOMMON
. $LIB
. $RC

tm_teststore() {
    local remote

    if test "$ISCSI_HOST"; then
	if test "$ISCSI_HOST_USER"; then
	    remote="$ISCSI_HOST_USER@$ISCSI_HOST"
	else
	    remote="$ISCSI_HOST"
	fi
	remote="ssh $remote"
    fi

    $remote sudo teststore "$@"
}

log "CLONE $*"

SRC_PATH=`arg_path $SRC`
DST_PATH=`arg_path $DST`
DST_DIR=`dirname $DST_PATH`
DST_HOST=`arg_host $DST`

if test -z "$SRC_PATH"; then
    log_error "no source for disk; aborting"
    exit 2
fi

if test -z "$SIZE"; then
    log_error "no size for disk; aborting"
    exit 2
fi

if ! [[ "$SRC_PATH" =~ ^([-_.a-zA-Z0-9]+|/dev/null)$ ]]; then
    log_error "invalid name for source"
    exit 2
fi

if ! get_diskid "$DST_PATH"; then
    log_error "can't get vm id or disk id"
    exit 1
fi

if ! vminfo=$($SI vm get $vm_id); then
    log_error "can't get VM info"
    exit 1
fi

if ! test "$vminfo"; then
    log_error "no info on VM"
    exit 1
fi

type=$(eval "$vminfo; echo \$vm_disks${vm_diskid}_type")

case ${type:=disk} in
    disk)
	SRCDIR="$DISKSRCDIR"
	;;
    *)
	log_error "disk types other than disk are not supported"
	exit 2
	# SRCDIR="$DISKSRCDIR"
	;;
esac

log "CLONE vmid=$vm_id diskid=$vm_diskid SRC_PATH=$SRC_PATH SIZE=$SIZE"

if test "$SRC_PATH" != /dev/null; then
    srcsize=$(stat -c %s $SRCDIR/$SRC_PATH)

    if test -z "$srcsize"; then
	log_error "can't get size of source (no such file?)"
	
	exit 1
    fi

    if test $(($SIZE * 1048576)) -lt $srcsize; then
	log_error "image size is bigger than disk size; can't clone"
	exit 2
    fi
fi

log "creating target"
url=$(tm_teststore create $SYSTEM $vm_id.$vm_diskid blah $SIZE)
error=$?
if ! test $error -eq 0; then
    log_error "creating target failed"
    exit 1
fi

if ! iscsi_parse_source iscsi://$ISCSI_PORTAL/$url/0; then
    log_error "can't parse iSCSI URL '$iscsi_url'"
    exit 1
fi

log "Creating directory $DST_DIR"
exec_and_log "mkdir -p $DST_DIR"

exec_and_log "ln -sf $iscsi_dev $DST_PATH"

if test $SRC_PATH != /dev/null; then

    if ! iscsi_login; then
	log_error "iSCSI login failed"
	exit 1
    fi

    log "transferring data"
    if ! sudo $ONE_DD $SRCDIR/$SRC_PATH $DST_PATH; then
	log_error "cloning failed"
	iscsi_logout
	exit 1
    fi
    log "data transfer done"

#    log "resizing image"
#    if ! sudo $RESIZE /dev/disk/by-path/$(readlink $iscsi_dev); then
#        log "image resize failed"
#     	iscsi_logout
#     	exit 1
#    fi
#    log "image resize done"

    iscsi_logout

fi
