#!/bin/bash

###############################################################################
#                                                                             #
#  Copyright (c) 2010 Nemzeti Információs Infrastruktúra Fejlesztési Intézet  #
#                                                                             #
#  Licensed under the Apache License, Version 2.0 (the "License");            #
#  you may not use this file except in compliance with the License.           #
#  You may obtain a copy of the License at                                    #
#                                                                             #
#    http://www.apache.org/licenses/LICENSE-2.0                               #
#                                                                             #
#  Unless required by applicable law or agreed to in writing, software        #
#  distributed under the License is distributed on an "AS IS" BASIS,          #
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
#  See the License for the specific language governing permissions and        #
#  limitations under the License.                                             #
#                                                                             #
###############################################################################

#set -e

SRC=$1
DST=$2

if [ -z "${ONE_LOCATION}" ]; then
    TMCOMMON=/usr/lib/one/mads/tm_common.sh
    LIB=/usr/lib/one/tm_commands/hostiscsi+nfs/iscsi-functions
    SI=/usr/lib/one/ruby/si/cloud_si
    RC=/etc/one/tm_hostiscsi+nfs/tm_hostiscsi+nfsrc
else
    TMCOMMON=$ONE_LOCATION/lib/mads/tm_common.sh
    LIB=$ONE_LOCATION/lib/tm_commands/hostiscsi+nfs/iscsi-functions
    SI=$ONE_LOCATION/lib/ruby/si/cloud_si
    PATH=/opt/one/bin:$PATH
    RC=$ONE_LOCATION/etc/tm_hostiscsi+nfs/tm_hostiscsi+nfsrc
fi

. $TMCOMMON
. $LIB
. $RC

log "LN $*"

SRC_PATH=`arg_path $SRC`
DST_PATH=`arg_path $DST`
DST_DIR=`dirname $DST_PATH`
DST_HOST=`arg_host $DST`

if test -z "$SRC_PATH"; then
    log_error "no source for disk; aborting"
    exit 2
fi

if ! [[ $SRC_PATH =~ ^[-_.a-zA-Z0-9]+$ ]]; then
    log_error "invalid name for source"
    exit 2
fi

if ! get_diskid $DST_PATH; then
    log_error "can't get vm id or disk id"
    exit 1
fi

if ! vminfo=$($SI vm get $vm_id); then
    log_error "can't get VM info"
    exit 1
fi

if ! test "$vminfo"; then
    log_error "no info on VM"
    exit 1
fi

type=$(eval "$vminfo; echo \$vm_disks${vm_diskid}_type")

case ${type:=disk} in
    cdrom)
	SRCDIR="$CDROMSRCDIR"
	;;
    *)
	log_error "disk types other than cdrom are not supported"
	exit 2
	# SRCDIR="$DISKSRCDIR"
	;;
esac

if ! test -f $SRCDIR/$SRC_PATH; then
    log_error "can't stat $SRCDIR/$SRC_PATH: no such file or directory"
    exit 2
fi

log "Creating directory $DST_DIR"
mkdir -p $DST_DIR

exec_and_log "ln -s $SRCDIR/$SRC_PATH $DST_PATH"
